#include "writeconf.h"


int	writexmlconf(	const char *romfile,		/* rom section */
				const char *snapshotfile,
				const char *hdma,
				const char *loadsnapshot,
				const char *windoweffects,
				const char *speedhacks,
				const char *forcehi,
				const char *forcelo,
				const char *interleaved,
				const char *interleaved2,
				const char *swaplayering,
				const char *cycles,			/* screen section */
				const char *snes9x,
				const char *forcentsc,
				const char *forcepal,
				const char *doublescreensize,
				const char *buffersize,	/* sound section */
				const char *usesound,
				const char *stereo,
				const char *usejoystick,	/* joystick section */
				const char *oldjoystick,
				const char *swapjoypads,
				const char *joy1dev,
				const char *joy2dev,
				const char *a,
				const char *b,
				const char *x,
				const char *y,
				const char *l,
				const char *r,
				const char *start,
				const char *select,
				char *conffilename
			){
	int newconffile=0;

	xmlDocPtr	doc;
	xmlNodePtr	tree,
				subtree;


	doc = xmlNewDoc("1.0");
	xmlSetDocCompressMode(doc,COMPRESSION);
	
	doc->children = xmlNewDocNode(doc, NULL, "goosnes", NULL);
		
		tree = xmlNewChild(doc->children, NULL, "rom", NULL); /* rom section */
			subtree = xmlNewChild(tree, NULL, "romfile", romfile);
			subtree = xmlNewChild(tree, NULL, "snapshotfile", snapshotfile);
			subtree = xmlNewChild(tree, NULL, "hdma", hdma);
			subtree = xmlNewChild(tree, NULL, "loadsnapshot", loadsnapshot);
			subtree = xmlNewChild(tree, NULL, "windoweffects", windoweffects);
			subtree = xmlNewChild(tree, NULL, "speedhacks", speedhacks);
			subtree = xmlNewChild(tree, NULL, "forcehi", forcehi);
			subtree = xmlNewChild(tree, NULL, "forcelo", forcelo);
			subtree = xmlNewChild(tree, NULL, "interleaved", interleaved);
			subtree = xmlNewChild(tree, NULL, "interleaved2", interleaved2);
			subtree = xmlNewChild(tree, NULL, "swaplayering", swaplayering);

		tree = xmlNewChild(doc->children, NULL, "screen", NULL);	/* screen section */
			subtree = xmlNewChild(tree, NULL, "cycles", cycles);
			subtree = xmlNewChild(tree, NULL, "snes9x", snes9x);
			subtree = xmlNewChild(tree, NULL, "forcentsc", forcentsc);
			subtree = xmlNewChild(tree, NULL, "forcepal", forcepal);
			subtree = xmlNewChild(tree, NULL, "doublescreensize", doublescreensize);

		tree = xmlNewChild(doc->children, NULL, "sound", NULL);	/* sound section */
			subtree = xmlNewChild(tree, NULL, "buffersize", buffersize);
			subtree = xmlNewChild(tree, NULL, "usesound", usesound);
			subtree = xmlNewChild(tree, NULL, "stereo", stereo);

		tree = xmlNewChild(doc->children, NULL, "joystick", NULL);	/* joystick section */
			subtree = xmlNewChild(tree, NULL, "usejoystick", usejoystick);
			subtree = xmlNewChild(tree, NULL, "oldjoystick", oldjoystick);
			subtree = xmlNewChild(tree, NULL, "swapjoypads", swapjoypads);
			subtree = xmlNewChild(tree, NULL, "joy1dev", joy1dev);
			subtree = xmlNewChild(tree, NULL, "joy2dev", joy2dev);
			subtree = xmlNewChild(tree, NULL, "a", a);
			subtree = xmlNewChild(tree, NULL, "b", b);
			subtree = xmlNewChild(tree, NULL, "x", x);
			subtree = xmlNewChild(tree, NULL, "y", y);
			subtree = xmlNewChild(tree, NULL, "l", l);
			subtree = xmlNewChild(tree, NULL, "r", r);
			subtree = xmlNewChild(tree, NULL, "start", start);
			subtree = xmlNewChild(tree, NULL, "select", select);


	if(NULL == conffilename){	/* save to default location */
		newconffile=1;
		conffilename=(char*)malloc(BUFSIZ*sizeof(char));
		assert(NULL != conffilename);
		get_conffilename(conffilename);
	}//if

	xmlSaveFile(conffilename,doc);
	if(newconffile){
			free(conffilename);
	}//if

	return 1;
}//writexmlconf

const char*	get_conffilename(char *buf){
		const char *home=getenv("HOME");
		FILE *bleh;

		if(NULL == home){
				strcpy(buf,"/etc/goosnes.conf");
		}//if(NULL == home)
		else{
				strncpy(buf,home,BUFSIZ);
				strncat(buf,"/.goosnes",BUFSIZ-strlen(buf)-1);
				if(NULL == (bleh=fopen(buf,"r"))){
						mkdir(buf,0755);	/* octal! */
						if(NULL == (bleh=fopen(buf,"r"))){
								printf("Creation of %s failed\n",
												buf);
								strcpy(buf,"/etc");
						}//if
						else fclose(bleh);
				}//if
				else fclose(bleh);

				strcat(buf,"/goosnes.conf");
		}//else

		return buf;
}//get_conffilename

int	writeconf(const GtkWidget *button,char *conffilename){
				char romfile[BUFSIZ];		/* rom section */
				char snapshotfile[BUFSIZ];
				char hdma[BUFSIZ];
				char loadsnapshot[BUFSIZ];
				char windoweffects[BUFSIZ];
				char speedhacks[BUFSIZ];
				char forcehi[BUFSIZ];
				char forcelo[BUFSIZ];
				char interleaved[BUFSIZ];
				char interleaved2[BUFSIZ];
				char swaplayering[BUFSIZ];
				char cycles[BUFSIZ];			/* screen section */
				char snes9x[BUFSIZ];
				char forcentsc[BUFSIZ];
				char forcepal[BUFSIZ];
				char doublescreensize[BUFSIZ];
				char buffersize[BUFSIZ];	/* sound section */
				char usesound[BUFSIZ];
				char stereo[BUFSIZ];
				char usejoystick[BUFSIZ];	/* joystick section */
				char oldjoystick[BUFSIZ];
				char swapjoypads[BUFSIZ];
				char joy1dev[BUFSIZ];
				char joy2dev[BUFSIZ];
				char a[BUFSIZ];
				char b[BUFSIZ];
				char x[BUFSIZ];
				char y[BUFSIZ];
				char l[BUFSIZ];
				char r[BUFSIZ];
				char start[BUFSIZ];
				char select[BUFSIZ];

				strncpy(romfile,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"rom"))),BUFSIZ);
				strncpy(snapshotfile,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"snapshot"))),BUFSIZ);

				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"hdma"))))
						strcpy(hdma,"y");
				else strcpy(hdma,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"loadsnapshot"))))
						strcpy(loadsnapshot,"y");
				else strcpy(loadsnapshot,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"windoweffects"))))
						strcpy(windoweffects,"y");
				else strcpy(windoweffects,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"speedhacks"))))
						strcpy(speedhacks,"y");
				else strcpy(speedhacks,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"hirom"))))
						strcpy(forcehi,"y");
				else strcpy(forcehi,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"lorom"))))
						strcpy(forcelo,"y");
				else strcpy(forcelo,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"interleaved"))))
						strcpy(interleaved,"y");
				else strcpy(interleaved,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"interleaved2"))))
						strcpy(interleaved2,"y");
				else strcpy(interleaved2,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"swaplayering"))))
						strcpy(swaplayering,"y");
				else strcpy(swaplayering,"n");

				strncpy(cycles,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"cycles"))),BUFSIZ);
				strncpy(snes9x,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"snes9x_entry"))),BUFSIZ);
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"forcentsc"))))
						strcpy(forcentsc,"y");
				else strcpy(forcentsc,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"forcepal"))))
						strcpy(forcepal,"y");
				else strcpy(forcepal,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"doublesize"))))
						strcpy(doublescreensize,"y");
				else strcpy(doublescreensize,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"soundon"))))
						strcpy(usesound,"y");
				else strcpy(usesound,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"stereo"))))
						strcpy(stereo,"y");
				else strcpy(stereo,"n");
				strncpy(buffersize,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"buffersize"))),BUFSIZ);

				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"usejoystick"))))
						strcpy(usejoystick,"y");
				else strcpy(usejoystick,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"oldjoystick"))))
						strcpy(oldjoystick,"y");
				else strcpy(oldjoystick,"n");
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"swapjoypads"))))
						strcpy(swapjoypads,"y");
				else strcpy(swapjoypads,"n");
				strncpy(joy1dev,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"joy1dev"))),BUFSIZ);
				strncpy(joy2dev,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"joy2dev"))),BUFSIZ);
				strncpy(a,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"a"))),BUFSIZ);
				strncpy(b,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"b"))),BUFSIZ);
				strncpy(x,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"x"))),BUFSIZ);
				strncpy(y,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"y"))),BUFSIZ);
				strncpy(l,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"l"))),BUFSIZ);
				strncpy(r,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"r"))),BUFSIZ);
				strncpy(start,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"start"))),BUFSIZ);
				strncpy(select,gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"select"))),BUFSIZ);

				return writexmlconf(	romfile,		/* rom section */
				snapshotfile,
				hdma,
				loadsnapshot,
				windoweffects,
				speedhacks,
				forcehi,
				forcelo,
				interleaved,
				interleaved2,
				swaplayering,
				cycles,			/* screen section */
				snes9x,
				forcentsc,
				forcepal,
				doublescreensize,
				buffersize,	/* sound section */
				usesound,
				stereo,
				usejoystick,	/* joystick section */
				oldjoystick,
				swapjoypads,
				joy1dev,
				joy2dev,
				a,
				b,
				x,
				y,
				l,
				r,
				start,
				select,
				conffilename);
}//writeconf
